IF OBJECT_ID('[ServerRoles_Extended]') IS NOT NULL DROP VIEW [ServerRoles_Extended]
GO

CREATE VIEW [ServerRoles_Extended]
AS
    SELECT SR.[ServerRole_Id]
          ,SR.[ServerRole_Name]
          ,SR.[Description_Text]
          ,[Servers_Count] = (SELECT COUNT(*)
                                FROM [ServerRoleServers] SRS 
                               WHERE SRS.[ServerRole_Id] = SR.[ServerRole_Id])

          ,[Environments_Count] = (SELECT COUNT([Environment_Id]) 
                                     FROM [ServerRoleServers] SRS 
                                INNER JOIN [EnvironmentServers] ES
                                        ON ES.[Server_Id] = SRS.[Server_Id]
                                    WHERE SR.[ServerRole_Id] = SRS.[ServerRole_Id])
                                    
          ,[Aggregate_ServerConfiguration_Status_Code] = CASE WHEN SC_COLLECTED.[Priority_Index] < 2 THEN 'D' WHEN SC_COLLECTED.[Priority_Index] >= 2 THEN 'C' ELSE 'U' END

          ,[Aggregate_Latest_CollectionRun_ExecutionStatus_Code] = E_BYSTATUS.[ExecutionStatus_Code]
          ,[Aggregate_Latest_CollectionRun_Execution_Id] = E_BYSTATUS.[Execution_Id]
          ,[Aggregate_Latest_CollectionRun_Start_Date] = E_BYSTATUS.[Start_Date]
		  ,[AllDriftPendingRemediation_Indicator] = CASE WHEN SR_DRIFT.[Priority_Index] = 1 THEN 'Y' ELSE 'N' END

           ,SR.[Raft_Id]
		   ,R.[Raft_Name]

		   ,SR.[CollectOperations_Csv]

	  FROM [ServerRoles] SR
	       LEFT JOIN [Rafts] R
	              ON R.[Raft_Id] = SR.[Raft_Id]
		   LEFT JOIN (SELECT [ServerRole_Id],
		                     [Priority_Index] = MIN(SC_DRIFT.[Priority_Index])
		                FROM [ServerRoleServers] SRS
					         LEFT JOIN (SELECT [Server_Id],
                                               [Priority_Index] = MIN([ServerConfiguration_Status_Priority])
                                          FROM [ServerConfigurations]
                                         GROUP BY [Server_Id]) SC_DRIFT
								    ON SC_DRIFT.[Server_Id] = SRS.[Server_Id]
					   GROUP BY SRS.[ServerRole_Id]) SR_DRIFT
				  ON SR_DRIFT.[ServerRole_Id] = SR.[ServerRole_Id]
		   LEFT JOIN (SELECT [CollectedFor_ServerRole_Id],
		                     [Priority_Index] = MIN([ServerConfiguration_Status_Priority])
					    FROM [ServerConfigurations]
					   GROUP BY [CollectedFor_ServerRole_Id]) SC_COLLECTED
				  ON SC_COLLECTED.[CollectedFor_ServerRole_Id] = SR.[ServerRole_Id]

OUTER APPLY (
    SELECT TOP 1 E.*
        FROM [Executions] E
            INNER JOIN (SELECT [Server_Id], [Execution_Id] = MAX([Execution_Id]) 
                            FROM [Executions]
                        GROUP BY [Server_Id]) E_LATEST
                    ON E.[Execution_Id] = E_LATEST.[Execution_Id]
            INNER JOIN [ServerRoleServers] _SRS
                    ON E.[Server_Id] = _SRS.[Server_Id]
        WHERE _SRS.[ServerRole_Id] = SR.[ServerRole_Id]
        ORDER BY CHARINDEX(E.[ExecutionStatus_Code], 'EWS')
) E_BYSTATUS
GO
