EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_AddMessage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Writes a message to the error log.'
GO

IF OBJECT_ID('[LogMessages_AddMessage]') IS NOT NULL DROP PROCEDURE [LogMessages_AddMessage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_AddMessage]
(
    @Message_Text VARCHAR(MAX),
    @Message_Level INT = NULL,
    @Category_Name VARCHAR(30) = NULL,
    @Details_Bytes VARBINARY(MAX) = NULL,
    @ShowNotification_Indicator YNINDICATOR = NULL,    
    @Job_Id INT = NULL,
    @Server_Id INT = NULL,
    @Environment_Id INT = NULL
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

     INSERT INTO [LogMessages]
     (
            [Logged_Date]
           ,[Message_Text]
           ,[Message_Level]
           ,[Category_Name]
           ,[Details_Bytes]
           ,[ShowNotification_Indicator]
           ,[Job_Id]
           ,[Server_Id]
           ,[Environment_Id]
     )
     VALUES
     (
            GETUTCDATE()
           ,@Message_Text
           ,COALESCE(@Message_Level, 30)
           ,@Category_Name
           ,@Details_Bytes
           ,COALESCE(@ShowNotification_Indicator, 'N')
           ,@Job_Id
           ,@Server_Id
           ,@Environment_Id
     );

    WITH LM AS (
        SELECT RN = ROW_NUMBER() OVER (PARTITION BY [Message_Level]
                                           ORDER BY [LogMessage_Id] DESC)
          FROM [LogMessages]
    )
    DELETE
      FROM LM
     WHERE RN > 1000

END TRY BEGIN CATCH
    EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_AddMessage] TO [OtterUser_Role]