EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_RecordConfiguration',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Records configuration data for a server.'
GO

IF OBJECT_ID('[Servers_RecordConfiguration]') IS NOT NULL DROP PROCEDURE [Servers_RecordConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

IF TYPE_ID('ServerConfigurationEntry') IS NOT NULL DROP TYPE ServerConfigurationEntry

CREATE TYPE ServerConfigurationEntry AS TABLE
(
	[ServerConfigurationType_Name] NVARCHAR(50) NOT NULL,
	[ServerConfigurationKey_Name] NVARCHAR(255) NOT NULL,
	[ServerConfiguration_Status_Code] CHAR(1) NOT NULL,
	[CollectedFor_ServerRole_Id] INT,
	[ServerConfiguration_Configuration] XML
)
GO

GRANT EXECUTE ON TYPE::[ServerConfigurationEntry] TO [OtterUser_Role]
GO

CREATE PROCEDURE [Servers_RecordConfiguration]
(
    @Server_Id INT,
	@Execution_Id INT,
	@RemediatedWith_Job_Id INT,
	@ServerConfigurationEntry_Table dbo.ServerConfigurationEntry READONLY
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
    BEGIN TRANSACTION

	-- use CTE as filter to prevent full table scan
	;WITH Filtered_SC
	AS
	(
		SELECT *
		  FROM [ServerConfigurations]
		 WHERE [Server_Id] = @Server_Id
	)
	MERGE Filtered_SC AS TARGET
	USING @ServerConfigurationEntry_Table AS SOURCE
	   ON (
	           TARGET.[Server_Id] = @Server_Id
	       AND TARGET.[ServerConfigurationType_Name] = SOURCE.[ServerConfigurationType_Name]
		   AND TARGET.[ServerConfigurationKey_Name] = SOURCE.[ServerConfigurationKey_Name]
		  )

	 WHEN MATCHED THEN
		UPDATE SET TARGET.[ServerConfiguration_Status_Code] = SOURCE.[ServerConfiguration_Status_Code],
		           TARGET.[CollectedFor_ServerRole_Id] = SOURCE.[CollectedFor_ServerRole_Id],
				   TARGET.[ServerConfiguration_Configuration] = SOURCE.[ServerConfiguration_Configuration],
				   TARGET.[CollectedOn_Execution_Id] = @Execution_Id,
				   TARGET.[RemediatedWith_Job_Id] = @RemediatedWith_Job_Id

	 WHEN NOT MATCHED BY TARGET THEN
		INSERT ([Server_Id], [ServerConfigurationType_Name], [ServerConfigurationKey_Name], [ServerConfiguration_Status_Code], [CollectedOn_Execution_Id], [CollectedFor_ServerRole_Id], [RemediatedWith_Job_Id], [ServerConfiguration_Configuration])
		VALUES (@Server_Id, SOURCE.[ServerConfigurationType_Name], SOURCE.[ServerConfigurationKey_Name], SOURCE.[ServerConfiguration_Status_Code], @Execution_Id, SOURCE.[CollectedFor_ServerRole_Id], @RemediatedWith_Job_Id, SOURCE.[ServerConfiguration_Configuration])

	 WHEN NOT MATCHED BY SOURCE THEN
		DELETE;

    COMMIT
END TRY BEGIN CATCH
    IF XACT_STATE()<>0 ROLLBACK
    EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Servers_RecordConfiguration] TO [OtterUser_Role]
